// ********************************************************************
// *
// * Autor(en)   : ebs EDV-Beratung-Service, Heiko Eggers, 1995
// * Gendert    : vgl. nderungen
// *
// * Beschreibung: Funktion zum Aufsplitten von Pfadangaben
// *               und Verzeichnisverwaltungsfunktionen
// *
// * Regel       : - Slashes werden nicht zurckgegeben
// *               - Erster Array-Eintrag ist immer das Laufwerk
// *               - Letzter Array-Eintrag kann DateiName oder
// *                 letzter Teil-Pfad sein (mu der Aufrufer wissen)
// *
// * Beispiel    : \test\name\sonst\ebs_dat.exe
// *
// *               aReturn[01] := H:          // Laufwerk
// *               aReturn[02] := test        // Pfad Teil 1
// *               aReturn[03] := name        // Pfad Teil 2
// *               aReturn[03] := sonst       // Pfad Teil 3
// *               aReturn[05] := ebs_dat.exe // DateiName
// *
// * FREEWARE
// *
// ********************************************************************
// * nderungen  :
// *
// * 18/12/96-hee: -Funktion XVERZEICHNIS verbessert
// * 09/02/96-hee: -Funktionen erstellt
// ********************************************************************

#include "ebs_vlib.ch"

FUNCTION XSPLITPFAD(cPfadName)

   LOCAL aReturn:={}, aToken:={}
   LOCAL cTmpFld:=""
   LOCAL i, nPos:=0
   LOCAL lOk:=.F.

   IF VALTYPE(cPfadName) != "C"
      cPfadName := DISKNAME()+":"+DIRNAME()
   ENDIF

   cPfadName    := ALLTRIM(cPfadName)

   IF cPfadName == "\"             // Nur Slash bergeben

      AADD(aReturn,DISKNAME()+":") // Laufwerk

   ELSE

      IF AT(":",cPfadName) > 0
         lOk := .T.                // Laufwerksangabe vorhanden
      ENDIF

      IF AT("\",cPfadName) > 0
         lOk := .T.                // Pfadangabe vorhanden
      ENDIF

      IF lOk

         IF (nPos:=AT(":",cPfadName)) > 0
            cTmpFld   := SUBSTR(cPfadName,1,nPos) // ":" mitgeben
            cPfadName := SUBSTR(cPfadName,nPos+1) // Variable aktualisieren
         ELSE
            cTmpFld   := DISKNAME() + ":"
         ENDIF

         AADD(aReturn, cTmpFld)       // Laufwerksbezeichnung (immer 1.Array-Eintrag)

         cTmpFld      := ""           // Init

         // *** Teilpfade "\..." und DateiNamen einstellen

         aToken       := XTOKENARRAY(cPfadName,"\")
         FOR i:=1 TO LEN(aToken)
             AADD(aReturn,aToken[i])
         NEXT

      ELSE

         // *** Es wurde nur ein DateiName bergeben

         AADD(aReturn,DISKNAME()+":") // Laufwerk
         AADD(aReturn,cPfadName)      // Datei

      ENDIF

   ENDIF

RETURN(aReturn)

FUNCTION XPFADNAME(cPfad)

   LOCAL cReturn:=""

   cReturn := XTEILPFAD(cPfad,0)

RETURN(cReturn)

FUNCTION XTEILPFAD(cPfad,nBisPos)

   LOCAL aTeile:={}
   LOCAL cReturn:=""
   LOCAL nLen

   // ***
   // *
   // * Regeln:
   // *
   // * 1. bergeben wird ein Pfad (L:\ABC\XYZ\III\)
   // *
   // * 2. Teilpfade sind alle Sub-Pfad von Beginn des
   // *    Pfad-Strings:
   // *
   // *    nBisPos= 0    L:\ABC\XYZ\III\
   // *    nBisPos=-1    L:\ABC\XYZ\
   // *    nBisPos=-2    L:\ABC\
   // *    nBisPos=-3    L:\
   // *
   // * 3. Eine groe Zahl -999 liefert immer L:\, also
   // *    die Laufwerksangabe inkl. BackSlash, ohne, da
   // *    man die genaue Lnge des Pfades kennen mu
   // *
   // * 4. Jede Positive Zahl liefert den Pfad zurck
   // *    (evtl. um letzten BackSlash ergnzt)
   // *
   // * 5. Dateien drfen im Pfad nicht vorkommen und werden
   // *    ggfs. durch die Funktion entfernt
   // *

   IF VALTYPE(cPfad) == "C"

      IF VALTYPE(nBisPos) != "N"
         nBisPos := 0
      ENDIF

      cReturn    := UPPER(cPfad)
      aTeile     := XSPLITPFAD(cReturn)    // Pfad zerlegen
      nLen       := LEN(aTeile)            // Lnge feststellen

      IF AT(".EXE",aTeile[nLen]) > 0 .OR. ;
         AT(".COM",aTeile[nLen]) > 0 .OR. ;
         AT(".BAT",aTeile[nLen]) > 0

         // *** Dateinamen entfernen

         ADEL(aTeile,nLen)                 // Dateiangabe entfernen
         cReturn := XMAKEPFAD(aTeile)      // Pfad zusammensetzen

      ENDIF

      IF nBisPos < 0

         cReturn := XMAKEPFAD(aTeile,nBisPos)

      ELSE

         // *** Vollstndigen Pfad inkl. letztem BackSlash zurckgeben

         cReturn := STRTRAN(ALLTRIM(cReturn)+"\","\\","\")

      ENDIF

   ENDIF

RETURN(cReturn)

FUNCTION XMAKEPFAD(aTeile,nBis)

   LOCAL i, nLen
   LOCAL cReturn

   // *** Einen Pfad wieder zusammensetzen.
   // *** Beispiel:   Wenn aTeile folgendermaen definiert ist
   // ***             und nBis entsprechend angegeben ist:
   // ***
   // ***             [1]   -> H:
   // ***             [2]   -> TOOLS
   // ***             [3]   -> EBS
   // ***             [4]   -> TEST.EXE
   // ***
   // ***             Dann ist liefert:
   // ***
   // ***             XMAKEPFAD(aTeile, 0)   -> H:\TOOLS\EBS\TEST.EXE
   // ***             XMAKEPFAD(aTeile,-1)   -> H:\TOOLS\EBS\
   // ***             XMAKEPFAD(aTeile,-2)   -> H:\TOOLS\
   // ***             XMAKEPFAD(aTeile,-3)   -> H:\
   // ***             XMAKEPFAD(aTeile,-999) -> H:\
   // ***

   IF VALTYPE(nBis) != "N"
      nBis := 0             // Alle Teile des Arrays benutzen, um den
   ENDIF                    // Pfad zu erstellen

   IF nBis >  0
      nBis := 0             // Parameter darf nicht grer Null sein
   ENDIF

   nLen    := LEN(aTeile)   // Lnge ermitteln
   cReturn := aTeile[1]     // X:   Laufwerksangabe

   FOR i:=2 TO nLen+nBis    // nBis addieren, da negativ bergeben
       IF EMPTY(aTeile[i])
          EXIT
       ELSE
          cReturn += "\" + aTeile[i]
       ENDIF
   NEXT

   cReturn := UPPER(ALLTRIM(STRTRAN(cReturn+"\","\\","\")))

RETURN(cReturn)

FUNCTION XFILENAME(cPfad)

   LOCAL aArr:={}
   LOCAL cReturn:=""

   // *** DateiNamen am Pfadende ermitteln und zurckgeben
   // *** Beispiel:
   // *** XFILENAME("H:\PROD\GICOM\GICOM.EXE") --> "GICOM.EXE"

   IF VALTYPE(cPfad) == "C"
      aArr    := XSPLITPFAD(cPfad)
      cReturn := aArr[LEN(aArr)]
   ENDIF

RETURN(cReturn)

FUNCTION XTEST_PFAD(cPfad)

   LOCAL cReturn:="", cRest
   LOCAL i:=0
   LOCAL lFound:=.F.

   IF VALTYPE(cPfad) == "C"
      cReturn := cPfad       // Inititialisierung Returnwert
   ENDIF

   IF !XVERZEICHNIS(cPfad)

      cRest  := SUBSTR(cPfad,2)
      lFound := .F.

      // *** Vorhandene Laufwerke durchsuchen (Z,...,C). Disketten-
      // *** Laufwerke A, B werden ignoriert

      FOR i:=90 TO 67 STEP -1
          cReturn := CHR(i)+cRest
          IF XVERZEICHNIS(cReturn)
             lFound := .T.
             EXIT
          ENDIF
      NEXT

      IF !lFound
         cReturn := ""
      ENDIF

   ENDIF

RETURN(cReturn)

FUNCTION XVERZEICHNIS(cDir)

   LOCAL lReturn :=.F.
   LOCAL cPruef:=""

   IF LEN(cDir) == 3 .AND. RIGHT(cDir,2) == ":\"

      lReturn    := .T.  // x:\ wird als Directory interpretiert

   ELSE

      // *** Den evtl. vorhandenen letzten Slash entfernen, weil
      // *** x:\abc\ nicht als Directory erkannt wird, sondern nur
      // *** x:\abc

      IF RIGHT(cDir,1) == "\"
         cPruef  := SUBSTR(cDir,1,LEN(cDir)-1)
      ELSE
         cPruef  := cDir
      ENDIF

      IF SUBSTR(XBITMAP(FILEATTR(cPruef)),4,1) == "1"
         lReturn := .T.  // Bei der Angabe handelt es sich um ein Directory
      ENDIF

   ENDIF

RETURN(lReturn)

FUNCTION XDIRECTORY(cVerz, cAttr, aOhne)

   LOCAL aReturn:={}, aDir:={}
   LOCAL nLen:=0

   aDir := DIRECTORY(cVerz, cAttr)

   IF VALTYPE(aOhne) == "A"

      // *** Es sollen Eintrge ausgeschlossen werden; die Steuerung
      // *** geschieht ber die FileExtension (EXE, BAT, TXT, ...)

      nLen := LEN(aDir)

      FOR i:=1 TO nLen

          IF AT(".",aDir[i][1]) > 0
             // *** Extension existiert; jetzt nachschlagen, ob
             // *** Dateien diesen Typs ausgeschlossen werden sollen
             IF ASCAN(aOhne,UPPER(RIGHT(aDir[i][1],3))) <= 0
                AADD(aReturn, aDir[i])
             ENDIF
          ELSE
             AADD(aReturn, aDir[i])
          ENDIF

      NEXT

   ELSE

      aReturn := aDir

   ENDIF

RETURN(aReturn)

FUNCTION XMAKEDIR(cSource)

   LOCAL aArr:={}
   LOCAL cPfad
   LOCAL nLen:=0

   IF VALTYPE(cSource) == "C"

      // *** cSource darf nur Directory-Angaben enthalten, d.h.
      // *** ein konkreter DateiName ist nicht mit zu bergeben!

      aArr  := XSPLITPFAD(cSource)
      nLen  := LEN(aArr)

      cPfad := aArr[1]             // X:   Laufwerksangabe

      FOR i:=2 TO nLen
          IF EMPTY(aArr[i])
             EXIT
          ELSE
             cPfad += "\" + aArr[i]
             DIRMAKE(cPfad)
          ENDIF
      NEXT

   ENDIF

RETURN(0)
