// ********************************************************************
// *
// * Autor(en)   : ebs EDV-Beratung-Service, Heiko Eggers, 1995
// * Gendert    : vgl. nderungen
// *
// * Beschreibung: Funktion zum Aktualisieren eines Datensatzes
// *
// * Beispiel    : SELECT h1
// *               cAlias := ALIAS()
// *               DO WHILE !EOF()
// *                  SELECT h2
// *                  XUPDATE(SELECT(),cAlias)
// *                  SELECT h1
// *                  SKIP
// *               ENDDO
// *
// * FREEWARE
// *
// ********************************************************************
// * nderungen  :
// *
// * 22/05/97-hee: -Beispielbeschreibung hinzugefgt
// * 14/02/97-hee: -Update eines Feldes als separate Function programmiert
// * 18/12/95-hee: -Funktion erstellt
// ********************************************************************

#include "ebs_vlib.ch"

FUNCTION XUPDATE(nSel, cA, lNeuanlage, lBlankUpdaten)

   LOCAL   i, nSaveSelect, nLen:=0, nReturn:=0
   LOCAL   cAlias
   LOCAL   aFeld:={}

   PRIVATE cTmpNeu

   // *** 1. Die nSel-Datei wird aktualisiert.
   // *** 2. Aus welcher DBF ergnzt werden soll, wird ber
   // ***    den bergebenen AliasNamen cA entschieden.
   // *** 3. Vorausgesetzt wird, da beide Dateien diesselbe
   // ***    Feld-Struktur haben.
   // *** 4. Alle Felder genau eines Satzes werden verarbeitet.
   // *** 5. bergabeparameter entscheiden, ob eine Neuanlage
   // ***    erfolgen soll und ob Felder mit Blankinhalten
   // ***    berschrieben werden drfen.
   // *** 6. Die bearbeitete/erstellte SatzNr wird zurckgeliefert

   IF VALTYPE(lNeuanlage)    != "L"
      // *** Defaultmig Neuanlage gewnscht
      lNeuanlage    := .T.
   ENDIF

   IF VALTYPE(lBlankUpdaten) != "L"
      // *** Defaultmig werden Eintrge auch mit Blank
      // *** berschrieben
      lBlankUpdaten := .T.
   ENDIF

   nSaveSelect := SELECT()

   SELECT (nSel)

   cAlias      := ALIAS()     // AliasNamen bereitstellen
   aFeld       := DBSTRUCT()  // DBF-Felder in Array stellen
   nLen        := LEN(aFeld)  // Anzahl Felder in der Struktur

   IF lNeuanlage
      DBAPPEND()
   ENDIF

   nReturn     := RECNO()     // SatzNr des Update-Records

   XRLOCK()

   FOR i:=1 TO nLen

       cTmpNeu := aFeld[i][1]

       IF lBlankUpdaten
          (cAlias)->&cTmpNeu    := (cA)->&cTmpNeu
       ELSE
          IF !EMPTY( (cA)->&cTmpNeu )
             (cAlias)->&cTmpNeu := (cA)->&cTmpNeu
          ENDIF
       ENDIF

   NEXT

   DBUNLOCK()

   SELECT (nSaveSelect)

RETURN(nReturn)

FUNCTION UpdField(nSel,aVon,aNach,bBlock)

   LOCAL  i, nSaveSelect, nLen:=0
   LOCAL  cAlias, cV, cN

   // *** Diese Funktion aktualisiert ein oder mehrere Feld(er) (Nach)
   // *** ausgehend von ein(em) Defeaultwert(en) (Von) ber eine UDF
   // *** Beispiel: SELECT 111
   // ***           UpdField(111,{"name"},{"sound"},{|a|SOUNDEX(a)})

   IF VALTYPE(bBlock) == "B"

      nSaveSelect := SELECT()

      SELECT (nSel)
      GO TOP

      cAlias      := ALIAS()
      nLen        := LEN(aVon)

      // *** Prfen, ob die angegebenen Felder existieren

      FOR i:=1 TO nLen
          cV      := aVon[i]
          cN      := aNach[i]
          IF FIELDPOS((cAlias)->&(cN)) > 0 .AND. ;
             FIELDPOS((cAlias)->&(cV)) > 0
          ELSE
             XPROTOK("Quell- und/oder Zielfeld existiert nicht")
             GO BOTT  // EOF herbeifhren, damit folgendes
             SKIP     // do while...enddo nicht ausgefhrt wird
             EXIT
          ENDIF
      NEXT

      DO WHILE !EOF()

         FOR i:=1 TO nLen

             cV              := aVon[i]
             cN              := aNach[i]

             (cAlias)->&(cN) := EVAL(bBlock,(cAlias)->&(cV))

         NEXT

         SKIP

      ENDDO

      SELECT (nSaveSelect)

   ELSE

      XPROTOK("UserDefineFunction (CodeBlock) fehlt")

   ENDIF

RETURN(0)
